@@ -24,6 +24,8 @@
 #include <gsf/gsf-doc-meta-data.h>
 #include <gsf/gsf-docprop-vector.h>
 #include <gsf/gsf-impl-utils.h>
+#include <string.h>
+#include <stdlib.h>
 
 struct _GsfDocMetaData {
 	GObject	base;
@@ -62,16 +64,16 @@ gsf_doc_meta_data_class_init (GObjectClass *gobject_class)
 	parent_class = g_type_class_peek_parent (gobject_class);
 }
 
-GSF_DYNAMIC_CLASS(GsfDocMetaData, gsf_doc_meta_data,
-		  gsf_doc_meta_data_class_init, gsf_doc_meta_data_init,
-		  G_TYPE_OBJECT)
+GSF_CLASS(GsfDocMetaData, gsf_doc_meta_data,
+	  gsf_doc_meta_data_class_init, gsf_doc_meta_data_init,
+	  G_TYPE_OBJECT)
 
 /**********************************************************************/
 
 /**
  * gsf_doc_meta_data_new :
  *
- * Returns a new metadata property collection
+ * Returns: a new metadata property collection
  **/
 GsfDocMetaData *
 gsf_doc_meta_data_new (void)
@@ -84,7 +86,7 @@ gsf_doc_meta_data_new (void)
  * @meta : #GsfDocMetaData
  * @name :
  *
- * Returns the property with name @id in @meta.  The caller can modify the
+ * Returns: the property with name @id in @meta.  The caller can modify the
  * property value and link but not the name.
  **/
 GsfDocProp *
@@ -98,7 +100,7 @@ gsf_doc_meta_data_lookup (GsfDocMetaData const *meta, char const *name)
 /**
  * gsf_doc_meta_data_insert :
  * @meta : #GsfDocMetaData
- * @name :
+ * @name : the id.
  * @value : #GValue
  *
  * Take ownership of @name and @value and insert a property into @meta.
@@ -167,26 +169,61 @@ gsf_doc_meta_data_store (GsfDocMetaData *meta, GsfDocProp *prop)
 	g_hash_table_replace (meta->table, prop->name, prop);
 }
 
+static void
+cb_collect_pairs (char *prop_name, GsfDocProp *prop, GPtrArray *pairs)
+{
+	g_ptr_array_add (pairs, prop_name);
+	g_ptr_array_add (pairs, prop);
+}
+
+static int
+deref_strcmp (const char **a, const char **b)
+{
+	return strcmp (*a, *b);
+}
+
 /**
  * gsf_doc_meta_data_foreach :
  * @meta : the collection
  * @func : the function called once for each element in the collection
- * @user_data : any supplied user data or NULL
+ * @user_data : any supplied user data or %NULL
  *
  * Iterate through each (key, value) pair in this collection
  **/
 void
 gsf_doc_meta_data_foreach (GsfDocMetaData const *meta, GHFunc func, gpointer user_data)
 {
+	GPtrArray *pairs;
+	unsigned ui;
+
 	g_return_if_fail (IS_GSF_DOC_META_DATA (meta));
-	g_hash_table_foreach (meta->table, func, user_data);
+
+	if (g_hash_table_size (meta->table) == 0)
+		return;
+
+	/* Sort the pairs by property name in order to generate consistent
+	   files. */
+	pairs = g_ptr_array_new ();
+	g_hash_table_foreach (meta->table, (GHFunc)cb_collect_pairs, pairs);
+
+	qsort (&g_ptr_array_index (pairs, 0),
+	       pairs->len / 2,
+	       2 * sizeof (gpointer),
+	       (GCompareFunc)deref_strcmp);
+
+	for (ui = 0; ui < pairs->len; ui += 2)
+		func (g_ptr_array_index (pairs, ui),
+		      g_ptr_array_index (pairs, ui + 1),
+		      user_data);
+
+	g_ptr_array_free (pairs, TRUE);
 }
 
 /**
  * gsf_doc_meta_data_size :
  * @meta : the collection
  *
- * Returns the number of items in this collection
+ * Returns: the number of items in this collection
  **/
 gsize
 gsf_doc_meta_data_size (GsfDocMetaData const *meta)
@@ -227,7 +264,7 @@ gsf_doc_meta_dump (GsfDocMetaData const *meta)
  * gsf_doc_prop_new :
  * @name :
  *
- * Returns a new #GsfDocProp which the caller is responsible for freeing.
+ * Returns: a new #GsfDocProp which the caller is responsible for freeing.
  * Takes ownership of @name.
  **/
 GsfDocProp *
@@ -249,7 +286,7 @@ gsf_doc_prop_new  (char *name)
  * gsf_doc_prop_free :
  * @prop : #GsfDocProp
  *
- * If @prop is non NULL free the memory associated with it
+ * If @prop is non %NULL free the memory associated with it
  **/
 void
 gsf_doc_prop_free (GsfDocProp *prop)
@@ -270,7 +307,7 @@ gsf_doc_prop_free (GsfDocProp *prop)
  * gsf_doc_prop_get_name :
  * @prop : #GsfDocProp
  *
- * Returns the name of the property, the caller should not modify the result.
+ * Returns: the name of the property, the caller should not modify the result.
  **/
 char const *
 gsf_doc_prop_get_name (GsfDocProp const *prop)
@@ -283,7 +320,7 @@ gsf_doc_prop_get_name (GsfDocProp const *prop)
  * gsf_doc_prop_get_val :
  * @prop : the property
  *
- * Returns the value of the property, the caller should not modify the result.
+ * Returns: the value of the property, the caller should not modify the result.
  **/
 GValue const *
 gsf_doc_prop_get_val (GsfDocProp const *prop)
@@ -318,7 +355,7 @@ gsf_doc_prop_set_val (GsfDocProp *prop, GValue *val)
  * @prop : #GsfDocProp
  * @val  : #GValue
  *
- * Returns the current value of @prop, and replaces it with @val
+ * Returns: the current value of @prop, and replaces it with @val
  * 	Caller is responsible for unsetting and freeing the result.
  **/
 GValue *
@@ -336,7 +373,7 @@ gsf_doc_prop_swap_val (GsfDocProp *prop, GValue *val)
  * gsf_doc_prop_get_link :
  * @prop : #GsfDocProp
  *
- * Returns the current link descriptor of @prop.  The result should not be
+ * Returns: the current link descriptor of @prop.  The result should not be
  * 	freed or modified.
  **/
 char const *
@@ -349,7 +386,7 @@ gsf_doc_prop_get_link (GsfDocProp const *prop)
 /**
  * gsf_doc_prop_set_link :
  * @prop : #GsfDocProp
- * @link : optionally NULL
+ * @link : optionally %NULL
  *
  * Sets @prop's link to @link
  **/
